import { ISettings, Ii18nTexts } from "./Settings";

export abstract class ConfigUI {
    settings: ISettings;
    $activationMessage: JQuery;
    $activationPanel: JQuery;
    $activationSpinner: JQuery;

    constructor(settings: ISettings) {
        this.settings = settings;
        this.$activationPanel = $("#activation-panel");
        this.$activationMessage = $("#activation-message", this.$activationPanel);
        this.$activationSpinner = $(".spinner", this.$activationPanel);
    }
    abstract async init(values: any);
    public static readonly ClientIdentityAuth: string = "Basic c2l0ZXZpc2lvbjo1V0IzZUsldlpqVWU=";
    hideLoader() {
        $("#loader").hide();
    }

    showLoader() {
        $("#loader").show();
    }
    logResult(type: string, message: string | IError | IMessage) {
        var msg = type + "\t" + this.toString(message);
        console.log(msg);
    }

    hideAlert() {
        $("#alert").removeClass("alert-success").removeClass("alert-danger").removeClass("alert-warning").removeClass("alert-info").hide();
    }
    i18n<T extends keyof Ii18nTexts>(key: T, ...args: any[]): string {
        if (!ImageVault.Util.has(this.settings.i18n, key)) return "[i18n missing for '" + key + "']";
        var val = this.settings.i18n[key].toString();
        val = ImageVault.Util.formatString(val, ...args);
        return val;
    }
    showActivationMessage(message: string) {
        this.$activationSpinner.show();
        this.$activationMessage.removeClass("alert-danger");
        this.$activationMessage.addClass("alert-info");
        this.$activationMessage.html(message);
        this.$activationPanel.show();
    }
    showActivationError(message: string) {
        this.$activationSpinner.hide();
        this.$activationMessage.removeClass("alert-info");
        this.$activationMessage.addClass("alert-danger");
        this.$activationMessage.html(message);
        this.$activationPanel.show();
    }
    hideActivationMessage() {
        this.$activationPanel.hide();
    }
    alert(message: string, type?: string) {
        this.hideAlert();
        if (!type) type = "danger";
        $("#alert").addClass("alert-" + type).html(message).show();
    }
    toString(arg: string | IError | IMessage): string {
        if (!arg) return "";
        if (typeof arg === "string") return arg as string;
        var e = arg as IError;
        if (e.error) {
            return this.toString(e.error);
        }
        var m = arg as IMessage;
        if (m.message) return m.message;
        return "";
    }
    error(arg1: string | IError | IMessage, arg2?: string | IError | IMessage) {
        //shift arguments down
        var msg = this.toString(arg1);
        var suffix = this.toString(arg2);
        if (suffix) {
            msg = msg + " " + suffix;
        }
        this.logResult("ERROR", msg);
        this.alert(msg);
    }

    getInput(key: string): JQuery {

        let $input: JQuery;
        $input = $('input[name=' + key + ']');
        //try with selects instead
        if ($input.length === 0)
            $input = $('select[name=' + key + ']');

        if ($input.length === 0) throw this.i18n("e-001-no-input-named-x-found", key);
        return $input;
    }
    setValue(key: string | JQuery, value: any): void {
        if (value && typeof value === "object") {
            value = JSON.stringify(value);
        }
        let $input: JQuery;
        if (typeof key === "string") {
            $input = this.getInput(key);
        } else {
            $input = key;
        }

        //radio
        if ($input.length > 1) {
            $input.prop("checked", false);
            $input.filter("[value='" + value + "']").prop("checked", true);
        } else {
            if($input.is("[type=checkbox]")) {
                $input.prop("checked",!!value);
            } else {
                $input.val(value);
            }
        }
    }
    getValue(key: string | JQuery): string {
        let $input: JQuery;
        if (typeof key === "string") {
            $input = this.getInput(key);
        } else {
            $input = key;
        }
        //this might be if input is a radio 
        if ($input && $input.length > 1) {
            $input = $input.filter(":checked");
        }

        if (!$input || !$input.length)
            return null;
        var v = $input.val();
        if (v == null) return null;
        return v.toString();
    }
    getIntValue(key: string | JQuery): number {
        let v = this.getValue(key);
        if (v == null || v.length === 0) return null;
        return parseInt(v);
    }
    getBooleanValue(key: string | JQuery): boolean {
        let $i = key as JQuery;
        if ((typeof key) === "string") {
            $i = this.getInput(key as string);
        }
        if (!$i) throw "Cannot find input named " + key;
        if ($i.attr("type").toLowerCase() !== "checkbox") {
            throw "Cannot get boolean value from a non checkbox input";
        }
        return $i.is(":checked");
    }
    getMedia(parameters: ImageVault.MediaItemQuery): Promise<ImageVault.WebMedia> {
        var self = this;
        return new Promise<ImageVault.WebMedia>((resolve, reject) => {
            ImageVault.MediaService.Find(self.settings.client, parameters).then((mediaItems) => {
                if (!mediaItems || !mediaItems.length) {
                    reject("ImageVault did not get a hit on the requested media");
                    return;
                }
                const mediaItem = mediaItems[0];
                const conversions = mediaItem.MediaConversions;
                if (!conversions || !conversions.length) {
                    reject("ImageVault did not get a conversion on the media item with id " + mediaItem.Id);
                    return;
                }
                var media = conversions[0] as ImageVault.WebMedia;
                resolve(media);
            }).catch(reject);
        });

    }


}
export interface IMessage {
    message: string;
}
export interface IError {
    error: string | IMessage;
}
