import { resolve } from "q";

export class DataStore {
    getAllMetadataDefinitions(): Promise<ImageVault.MetadataDefinition[]> {
        return this.allMetadataDefinitions;
    }
    private client: ImageVault.IClient;
    private allMetadataDefinitions: Promise<ImageVault.MetadataDefinition[]>;
    private allVaults: Promise<ImageVault.Vault[]>;
    private allCategories: Promise<ImageVault.Category[]>;
    private categoryIndex: Promise<{ [id: number]: ImageVault.Category }>;

    private version: Promise<string>;

    constructor(client: ImageVault.IClient) {
        this.client = client;
        var self = this;
        this.allMetadataDefinitions = new Promise<ImageVault.MetadataDefinition[]>((resolve) => {
            //get all definitions
            ImageVault.MetadataDefinitionService.Find(self.client, {}).then((mds) => {
                //sort by type
                mds.sort((m1, m2) => {
                    var typeDiff = m1.MetadataDefinitionType - m2.MetadataDefinitionType;
                    if (typeDiff) return typeDiff;
                    return m1.Name > m2.Name ? 1 : m1.Name < m2.Name ? -1 : 0;
                });
                resolve(mds);
            });
        });
        this.allVaults = ImageVault.VaultService.Find(self.client, {});
        this.allCategories = ImageVault.CategoryService.getCategories(self.client);
        this.categoryIndex = this.allCategories.then((list) => {
            var d: { [id: number]: ImageVault.Category } = {};
            var cs: ImageVault.Category[] = [];
            cs = cs.concat(list);
            while (cs.length) {
                var c = cs[0];
                cs = cs.slice(1);
                d[c.Id] = c;
                if (c.Categories && c.Categories.length) {
                    cs = cs.concat(c.Categories);
                }
            }
            return d;
        });
        this.version = new Promise<string>((resolve, reject) => {
            self.client.get<IInfoVersion>("info/version", (info) => {
                resolve(info.version);
            }, (reason) => { reject(reason) });
        });
    }
    getVersion(): Promise<string> {
        return this.version;
    }
    getAllCategories(): Promise<ImageVault.Category[]> {
        return this.allCategories;
    }
    async getCategoryById(id: number): Promise<ImageVault.Category> {
        var d = await this.categoryIndex;
        return d[id];
    }
    getVault(id: number): Promise<ImageVault.Vault> {
        var self = this;
        if (!id) return null;

        return new Promise((resolve) => {
            self.allVaults.then((vaults) => {
                var matchingVaults = vaults.filter((v) => { return v.Id === id });
                if (!matchingVaults || matchingVaults.length !== 1) resolve(null);
                resolve(matchingVaults[0]);
            });
        });
    }
    getMetadataDefinitionByTypeAndName(definitionType: ImageVault.MetadataDefinitionType, name: string): Promise<ImageVault.MetadataDefinition> {
        var self = this;
        return new Promise((resolve) => {
            self.allMetadataDefinitions.then((definitions) => {
                var matchingDefinitions = definitions.filter((md) => { return md.MetadataDefinitionType === definitionType && md.Name === name });
                if (!matchingDefinitions || matchingDefinitions.length !== 1) resolve(null);
                resolve(matchingDefinitions[0]);
            });
        });

    }
    getMetadataDefinition(id: number): Promise<ImageVault.MetadataDefinition> {
        var self = this;
        if (!id) return null;

        return new Promise((resolve) => {
            self.allMetadataDefinitions.then((definitions) => {
                var matchingDefinitions = definitions.filter((md) => { return md.Id === id });
                if (!matchingDefinitions || matchingDefinitions.length !== 1) resolve(null);
                resolve(matchingDefinitions[0]);
            });
        });
    }
    formatAssetMetadata(metadata: ImageVault.AssetMetadata): Promise<IFormattedMetadataInfo> {
        return new Promise<IFormattedMetadataInfo>((resolve) => {
            this.getMetadataDefinition(metadata.definitionId).then((md) => {
                var value = metadata.value;
                switch (md.MetadataType) {
                    case ImageVault.MetadataType.Boolean:
                        value = metadata.booleanValue ? "true" : "false";
                        break;
                    case ImageVault.MetadataType.DateTime:
                        value = new Date(metadata.dateValue).toLocaleString();
                        break;
                }
                resolve({ metadataDefinition: md, formattedValue: value });
            });
        });
    }
}
export interface IFormattedMetadataInfo {
    metadataDefinition: ImageVault.MetadataDefinition;
    formattedValue: string;
}
export interface IInfoVersion {
    version: string;
}