import * as $ from "jquery";
import { VersionUtil } from "./VersionUtil";
import { ConfigUI } from "./ConfigUI";
import { ISettings } from "./Settings";

export class GlobalConfigUI extends ConfigUI {
    static readonly imageVaultMinVersion: string = "5.9";
    static readonly ImageVaultUrlKey: string = "imageVaultUrl";
    $currentUserStatus: JQuery;
    $logoutButton: JQuery;
    $imageVaultUrlTextBox: JQuery;
    $requireAltTextCheckbox: JQuery;

    constructor(settings: ISettings) {
        super(settings);
    }
    public init() {
        var self = this;
        this.$currentUserStatus = $("#current-user-status");
        this.$logoutButton = $("#logout-user-button");
        this.$imageVaultUrlTextBox = this.getInput(GlobalConfigUI.ImageVaultUrlKey);
        this.$requireAltTextCheckbox = this.getInput("alt-text-required");
        this.setValue(this.$imageVaultUrlTextBox, this.settings.config.ivUrl);
        this.setValue(this.$requireAltTextCheckbox, this.settings.config.requireAltText || false);
        this.updateCurrentUserSettings();

        this.$imageVaultUrlTextBox.blur(() => {
            self.showLoader();
            self.hideAlert();
            setTimeout(() => {
                try {
                    let res = self.verifyImageVaultUrlSync();
                    if (!res.success) {
                        self.alert(res.message);
                    } else {
                        self.updateCurrentUserSettings();
                    }
                } finally {
                    self.hideLoader();
                }
            }, 100);
        });
        $("#clear-settings-button").click((e) => {
            e.preventDefault();
            self.showLoader();
            self.resetAppConfig();
            self.hideLoader();
            return false;
        });
        this.$logoutButton.click((e) => {
            e.preventDefault();
            var client = this.createClient();
            if (client) {
                client.removeCachedCredentials();
                this.updateCurrentUserSettings();
            }
            return false;
        });

        window.validate = () => {
            try {
                //TODO: loader won't show since we do synchronous ajax calls to support the SiteVision validate method that is synchronous
                self.showLoader();
                let res = self.verifyImageVaultUrlSync();
                if (!res.success) {
                    self.alert(res.message);
                    return false;
                }
                if (!self.verifyClientAuthentication()) {
                    self.alert(self.i18n("license-not-active-at-x", self.getValue(GlobalConfigUI.ImageVaultUrlKey)));
                    return false;
                }
                return true;
            } finally {
                self.hideLoader();
            }
        }
        window.getValues = () => {
            var url = this.getValue(this.$imageVaultUrlTextBox);
            var requireAltText = this.getBooleanValue(this.$requireAltTextCheckbox);
            window.requester.doPost({ route: '/config', data: { ivUrl: url, requireAltText: requireAltText }, async: false })
                .done((response) => { self.logResult("info", self.i18n("done") + ":" + JSON.stringify(response)); })
                .fail((response) => { self.logResult("error", "fail: " + JSON.stringify(response)); });

            return {};
        };
        $("#global-settings").show();

        this.hideLoader();
    }

    private createClient(): ImageVault.PinClient {
        try {
            var url = this.getValue(GlobalConfigUI.ImageVaultUrlKey);
            if (!url) return null;
            return new ImageVault.PinClient({ ivUrl: url, activationCallback: () => { }, clientIdentityAuth: ConfigUI.ClientIdentityAuth });
        } catch (e) {
            this.logResult("ERROR", e);
            return null;
        }
    }
    private resetAppConfig() {
        window.requester.doDelete({ route: '/config', async: false })
    }
    updateCurrentUserSettings() {
        var pinClient = this.createClient();
        if (pinClient && pinClient.hasCachedCredentials()) {
            this.$logoutButton.removeAttr("disabled");
            this.$currentUserStatus.text(this.i18n("cached-credentials-exists"));
        } else {
            this.$logoutButton.attr("disabled", "disabled");
            this.$currentUserStatus.text(this.i18n("no-cached-credentials-exists"));
        }
    }
    private verifyClientAuthentication(): boolean {
        var self = this;

        var pinClient = this.createClient();
        if (!pinClient) return false;
        let success = false;
        pinClient.pinCodeRequest(false, (pinCodeResponse) => {
            success = !!(pinCodeResponse.pin);
            self.logResult("INFO", "Pin code request succeeded(" + success + "): " + pinCodeResponse.pin);
        }, (error) => {
            self.logResult("ERROR", error);
            success = false;
        });
        return success;
    }
    private verifyImageVaultUrlSync(): VerifyVersionResult {
        var self = this;
        var url = this.getValue(GlobalConfigUI.ImageVaultUrlKey);
        var res = { success: false, message: this.i18n("you-must-supply-the-url-to-the-imagevault-service") };
        if (!url) return res;
        url = url.toLocaleLowerCase();
        if (!url.endsWith('/')) {
            url = url + "/";
        }
        this.setValue(GlobalConfigUI.ImageVaultUrlKey, url);
        $.ajax({
            async: false,
            method: 'GET',
            url: url + "/apiv2/info/version",
            dataType: 'json',
            success: function (info) {
                if (!info || !info.version) {
                    res.message = self.i18n("unsupported-imagevault-service-located-at", url);
                    return;
                }
                if (VersionUtil.verifyMinVersion(info.version, GlobalConfigUI.imageVaultMinVersion)) {
                    res.message = self.i18n("imagevault-service-verified-version", info.version);//"ImageVault service verified (version " + info.version + ")";
                    res.success = true;
                    return;
                }
                res.message = self.i18n("imagevault-service-does-not-meet-required-version", GlobalConfigUI.imageVaultMinVersion, url, info.version);
                return;
            },
            error: function (jqXHR, status, e) {
                var msg = e;
                if (jqXHR.responseJSON) {
                    msg = jqXHR.responseJSON.ExceptionMessage || jqXHR.responseJSON.Message || msg;
                }
                res.message = self.i18n("failed-to-connect-to-imagevault-service", url);
                console.log(msg);
            }
        });
        return res;

    }
}
class VerifyVersionResult {
    success: boolean;
    message: string;
}
