import { ConfigUI } from "./ConfigUI";
import { IInstanceBroker } from "./InstanceConfigUI";
import { MetadataTextUI } from "./MetadataTextUI";
import { ConfigPanelUI } from "./ConfigPanelUI";
import { ThumbSelectorUI } from "./ThumbSelectorUI";

export class InfoPanelUI extends ConfigUI {
    $removeButton: JQuery;
    $editButton: JQuery;
    $assetInfoList: JQuery<HTMLDListElement>;
    altTextUI: MetadataTextUI;
    captionUI: MetadataTextUI;
    currentAsset: ImageVault.IAsset;
    $mediaSettingsDocumentRow: JQuery;
    $mediaSettingsMediaRow: JQuery;
    widthSystemMetadata: ImageVault.MetadataDefinition;
    assetOptimizationPanelUI: ConfigPanelUI;
    broker: IInstanceBroker;
    assetFormatSelector: ThumbSelectorUI;
    puffFormatSelector: ThumbSelectorUI;

    constructor(broker: IInstanceBroker) {
        super(broker.settings);
        this.broker = broker;
    }
    async init() {
        var self = this;
        this.$editButton = $("#edit-asset-button");
        this.$removeButton = $("#remove-asset-button");
        this.$assetInfoList = $("#asset-info-list");
        this.$mediaSettingsDocumentRow = $("#media-settings-document");
        this.$mediaSettingsMediaRow = $("#media-settings-media");
        this.widthSystemMetadata = await this.settings.dataStore.getMetadataDefinitionByTypeAndName(ImageVault.MetadataDefinitionType.System, "ImageWidth");
        if (!this.widthSystemMetadata) {
            this.logResult("ERROR", "Cannot find System ImageWidth metadata definition");
        }
        if (!this.settings.instanceConfig.puff) {
            this.settings.instanceConfig.puff = { formatId: -1 };
        }
        this.puffFormatSelector = new ThumbSelectorUI(this.broker, { prefix: "puff", targetSize: 200, data: this.settings.instanceConfig.puff });
        this.assetFormatSelector = new ThumbSelectorUI(this.broker, {
            prefix: "asset", targetSize: 800, data: this.settings.instanceConfig, thumbEdited: () => {
                self.puffFormatSelector.updateThumbnail();
            }
        });
        this.puffFormatSelector.init();
        this.assetFormatSelector.init();

        this.altTextUI = new MetadataTextUI(this.settings, "alt-text", this.settings.instanceConfig.altTextData, this.settings.config.requireAltText);
        this.altTextUI.init();
        this.captionUI = new MetadataTextUI(this.settings, "caption", this.settings.instanceConfig.captionData, false);
        this.captionUI.init();
        this.assetOptimizationPanelUI = new ConfigPanelUI(this.settings, "asset-optimization-panel");
        this.assetOptimizationPanelUI.init();

        this.$editButton.click(function (e) {
            e.preventDefault();
            $(this).blur();
            self.assetFormatSelector.editAsset();
            return false;
        });
        this.$removeButton.click((e) => {
            e.preventDefault();
            $(this).blur();
            self.selectAsset(null);
            return false;
        });


        await this.assetFormatSelector.updateFormats();
        await this.puffFormatSelector.updateFormats();
        //update box to show current selected item
        await this.updateSelectedInfoById();

    }

    async updateSelectedInfoById(id?: number) {
        var self = this;
        if (!id) {
            id = this.settings.instanceConfig.id;
        }
        if (!id) {
            return;
        }
        try {
            var r = await this.broker.mediaServiceUtil.getAsset(id);
            if (!r.value || r.value.length === 0) {
                throw self.i18n("cannot-find-asset-with-id-x", id);
            }
            var asset = r.value[0];
            await self.updateSelectedInfo(asset);
        } catch (e) {
            await self.updateSelectedInfo(null);
            self.assetFormatSelector.showThumbnailLoaderErrorMessage(e);
        }
    }

    async selectAsset(asset: ImageVault.IAsset): Promise<void> {
        this.settings.instanceConfig.effects = null;
        var id = 0;
        if (asset) {
            id = parseInt(asset.id)
        };
        this.settings.instanceConfig.id = id;
        await this.updateSelectedInfo(asset);
    }



    /**
     * Checks if the current selected asset is editable.
     */
    public isCurrentAssetEditable(): boolean {
        return this.isCurrentAssetVideo() || this.isCurrentAssetImage();
    }
    public isCurrentAssetImage(): boolean {
        if (!this.currentAsset) return false;
        var contentType = this.currentAsset.contentType;
        if (!contentType) return false;
        contentType = contentType.toLocaleLowerCase();
        if (contentType.startsWith("image")) return true;
        if (contentType == "application/pdf") return true;
        if (contentType == "application/postscript") return true;
        return false;
    }
    public isCurrentAssetVideo(): boolean {
        if (!this.currentAsset) return false;
        var contentType = this.currentAsset.contentType;
        if (!contentType) return false;
        contentType = contentType.toLocaleLowerCase();
        if (contentType.startsWith("video")) return true;
        return false;
    }
    getCurrentAssetWidth(): number {
        var self = this;
        //cannot find width system metadata
        if (!this.widthSystemMetadata) {
            return null;
        }
        if (!this.currentAsset || !this.currentAsset.metadata) return null;
        var matchingMetadata = this.currentAsset.metadata.filter((m) => {
            return m.definitionId === self.widthSystemMetadata.Id;
        });
        if (!matchingMetadata || !matchingMetadata.length)
            return null;
        return matchingMetadata[0].integerValue;
    }

    private enableFormatSelection(enable: boolean = true) {
        this.assetFormatSelector.enable(enable);
        if (enable) {
            this.$mediaSettingsDocumentRow.hide();
            this.$mediaSettingsMediaRow.show();
        } else {
            this.$mediaSettingsMediaRow.hide();
            this.$mediaSettingsDocumentRow.show();
        }
    }
    async updateSelectedInfo(asset: ImageVault.IAsset) {
        this.currentAsset = asset;
        var self = this;
        this.assetFormatSelector.clearThumb();
        this.puffFormatSelector.clearThumb();
        this.$assetInfoList.empty();
        this.altTextUI.setAsset(asset);
        this.captionUI.setAsset(asset);
        if (!asset) {
            //format is enabled when no item is selected
            this.enableFormatSelection();
            this.altTextUI.setEnabled(true);
            this.captionUI.setEnabled(true);
            this.broker.assetInteractionUI.setEnabled(true);
            this.assetOptimizationPanelUI.setEnabled(true);
            self.$editButton.addClass("disabled");
            self.$removeButton.addClass("disabled");
            return;
        }
        //depending if the asset is editable or not
        let assetEditable = this.isCurrentAssetEditable();
        let assetImage = this.isCurrentAssetImage();
        this.enableFormatSelection(assetEditable);
        this.altTextUI.setEnabled(assetImage);
        this.captionUI.setEnabled(assetEditable);
        this.assetOptimizationPanelUI.setEnabled(assetEditable);
        //asset interaction is only available on images
        this.broker.assetInteractionUI.setEnabled(this.isCurrentAssetImage());
        this.assetFormatSelector.updateThumbnail();
        this.puffFormatSelector.updateThumbnail();

        if (assetEditable) {
            this.$editButton.removeClass("disabled");
        } else {
            self.$editButton.addClass("disabled");
        }
        this.$removeButton.removeClass("disabled");
        //todo: remove?
        this.createHeading(asset.name);
        this.addInfo("id", asset.id);
        var v = await this.settings.dataStore.getVault(asset.vaultId);
        self.addInfo(this.i18n("vault"), v.Name);
        var date = new Date(asset.modified);
        self.addInfo(this.i18n("uploaded"), date.toLocaleString());
        self.addInfo(this.i18n("uploaded-by"), asset.modifiedBy);
        //todo: remove?
        self.addInfo(this.i18n("content-type"), asset.contentType);
        //TODO:categories
        var categories = "";
        if (asset.categories && asset.categories.length) {
            categories = asset.categories.map((c) => { return c.name }).join(", ");
        }
        self.addInfo(this.i18n("categories"), categories);
        if (asset.metadata && asset.metadata.length) {
            var metadata = await Promise.all(asset.metadata.map(async (m) => {
                var fm = await self.settings.dataStore.formatAssetMetadata(m);
                return fm;
            }));

            for (const type in ImageVault.MetadataDefinitionType) {
                let data = metadata.filter(m => m.metadataDefinition.MetadataDefinitionType == Number(type))
                if (data.length) {
                    self.createHeading(ImageVault.MetadataDefinitionType[type]);
                    data.map((m) => {
                        self.addInfo(m.metadataDefinition.Name, m.formattedValue);
                    });
                };
            };
        };
    }
    createHeading(heading: string) {
        this.$assetInfoList.append("<dt class='heading'>" + heading + "</dt>");
    }
    addInfo(name: string, value: string) {
        this.$assetInfoList.append("<dt>" + ImageVault.Util.htmlEncode(name) + "</dt>");
        this.$assetInfoList.append("<dd>" + ImageVault.Util.htmlEncode(value) + "</dd>");
    }


}
