import { ISettings, IMetadataTextData } from "./Settings";
import { ConfigPanelUI } from "./ConfigPanelUI";

export class MetadataTextUI extends ConfigPanelUI {
    name: string;
    data: IMetadataTextData;
    private choiceKey: string;
    $choiceCustomDetails: JQuery<HTMLElement>;
    $choiceMetadataDetails: JQuery<HTMLElement>;
    $customValueInput: JQuery<HTMLElement>;
    $metadataDefinitionsSelect: JQuery<HTMLSelectElement>;
    $panel:JQuery<HTMLElement>;

    private asset: ImageVault.IAsset;
    $metadataValue: JQuery<HTMLElement>;
    required: boolean;

    constructor(settings: ISettings, name: string, data: IMetadataTextData, required: boolean) {
        super(settings, name + "-metadata-text-panel");
        this.name = name;
        this.required = required || false;
        this.data = data || { choice: "custom" };
        this.choiceKey = this.name + "-choice";
    }

    getNamedElement<T extends HTMLElement>(id: string): JQuery<T> {
        var identifier = this.name + "-" + id;
        var $j = $("#" + identifier);
        if ($j.length !== 1) {
            throw "cannot find DOM element with id " + identifier;
        }
        return $j as JQuery<T>;
    }
    init() {
        var self = this;
        this.$choiceCustomDetails = this.getNamedElement("choice-custom-details");
        this.$choiceMetadataDetails = this.getNamedElement("choice-metadata-details");
        this.$metadataDefinitionsSelect = this.getNamedElement<HTMLSelectElement>("from-metadata-id-select");
        this.$metadataValue = this.getInput(this.name + "-from-metadata-text");
        this.$customValueInput = this.getInput(this.name + "-custom");
        this.$panel = this.$customValueInput.closest("div.panel");

        //make sure we have a valid value for choice
        var choice = this.data.choice;
        if (choice !== "metadata" && choice !== "none") {
            choice = "custom";
        }

        //if required, do not allow none alternative
        if (this.required) {
            if (choice === "none")
                choice = "custom";
            //hide none alternative
            var $noneAlternativeRow = $("#"+this.name + "-choice-none").closest("div.row");
            $noneAlternativeRow.hide();
        }

        this.setValue(this.choiceKey, choice);
        this.setValue(this.$customValueInput, this.data.customValue);

        this.updateDetailsUI();
        this.updateMetadataSelect()
            .then(() => {
                if (self.data.metadataId) {
                    self.setValue(self.$metadataDefinitionsSelect, self.data.metadataId);
                }
            })
            .then(() => { return self.updateMetadataValue(); })
            .then(() => {
                self.$metadataDefinitionsSelect.change(() => {
                    self.saveSelectedMetadataDefinition();
                    self.updateMetadataValue().catch((e) => { self.error(self.i18n("e-003-failed-to-update-metadata-value"), e); });
                });
            }).catch((e) => {
                self.error(self.i18n("e-004-failed-to-retrieve-metadata-definitions"), e);
            });
        this.getInput(this.choiceKey).change(() => {
            self.updateDetailsUI();
        });
    }
    saveSelectedMetadataDefinition() {
        let id = this.getIntValue(this.$metadataDefinitionsSelect);
        this.data.metadataId = id;
    }
    setAsset(asset: ImageVault.IAsset): Promise<void> {
        this.asset = asset;
        return this.updateMetadataValue();
    }
    updateMetadataValue(): Promise<void> {
        var self = this;
        return new Promise((resolve) => {
            let id = self.data.metadataId;
            if (id && self.asset) {
                var matchingMetadata = self.asset.metadata.filter((m => m.definitionId === id));
                if (matchingMetadata && matchingMetadata.length > 0) {
                    self.settings.dataStore.formatAssetMetadata(matchingMetadata[0]).then((fm) => {
                        self.$metadataValue.val(fm.formattedValue);
                        resolve();
                    });
                    return;
                }
            }
            self.$metadataValue.val("");
            resolve();
        });
    }
    updateMetadataSelect(): Promise<void> {
        let self = this;
        let oldValue = this.data.metadataId;

        return new Promise((resolve) => {
            self.settings.dataStore.getAllMetadataDefinitions().then((metadataDefinitions) => {
                self.$metadataDefinitionsSelect.empty();
                metadataDefinitions.map((md) => {
                    let $o = $("<option></option>");
                    $o.attr("value", md.Id);
                    $o.text(ImageVault.MetadataDefinitionType[md.MetadataDefinitionType] + " - " + md.Name);
                    self.$metadataDefinitionsSelect.append($o);
                });
                self.setValue(self.$metadataDefinitionsSelect, oldValue);
                if (self.$metadataDefinitionsSelect[0].selectedIndex < 0) {
                    self.$metadataDefinitionsSelect[0].selectedIndex = 0;
                    self.saveSelectedMetadataDefinition();
                }
                resolve();
            });
        });
    }
    updateDetailsUI() {
        var choice = this.getValue(this.choiceKey);
        this.$choiceCustomDetails.hide();
        this.$choiceMetadataDetails.hide();

        switch (choice) {
            case "custom":
                this.$choiceCustomDetails.show();
                break;
            case "metadata":
                this.$choiceMetadataDetails.show();
                break;
        }
    }
    getData(): IMetadataTextData {

        var choice = this.getValue(this.choiceKey);
        var value = "";
        var metadataValue = this.getValue(this.$metadataValue);
        var customValue = this.getValue(this.$customValueInput);
        let id = this.data.metadataId;
        switch (choice) {
            case "custom":
                value = customValue;
                break;
            case "metadata":
                value = metadataValue;
                break;
            case "none":
                value = "";
                break;
        }
        if(this.isEnabled() && this.required && !value) {
            this.$panel.addClass("has-error");
            throw this.i18n('metadata-text-value-required');
        } else {
            this.$panel.removeClass("has-error");
        }
        return { choice: choice, customValue: customValue, metadataId: id, value: value };

    }
}