import { ConfigUI } from "./ConfigUI";
import { IInstanceBroker } from "./InstanceConfigUI";
import { CategoryFilterUI } from "./CategoryFilterUI";
import { VersionUtil } from "./VersionUtil";

export class SearchPanelUI extends ConfigUI {

    currentSearchPage: number = 0;
    moreSearchPagesExists: boolean = false;
    $searchButton: JQuery;
    $assets: JQuery;
    $assetsDiv: JQuery;
    $searchLoader: JQuery;
    $searchTextBox: JQuery;
    searchHits: ImageVault.IAsset[];
    $searchOrderBySelect: JQuery;
    overrideDefaultOrderBy: string;
    broker: IInstanceBroker;
    categoryFilterUI: CategoryFilterUI;

    constructor(broker: IInstanceBroker) {
        super(broker.settings);
        this.broker = broker;
    }
    async init() {

        var self = this;
        this.$assets = $("#assets");
        this.$assetsDiv = $("#assets-div");
        this.$searchLoader = $("#search-loader");
        this.$searchTextBox = this.getInput("search");
        this.$searchOrderBySelect = this.getInput("searchOrderBy");
        this.$searchButton = $("#search");
        this.categoryFilterUI = new CategoryFilterUI(this.broker.settings, () => {
            self.updateHeight();
            self.search(false);
        });

        this.categoryFilterUI.init();
        //reset form
        this.setValue(this.$searchTextBox, "");
        this.setValue(this.$searchOrderBySelect, this.getDefaultSearchOrder());

        //hook up events
        this.$searchOrderBySelect.change(() => {
            self.setSearchOrderBy(self.getValue(self.$searchOrderBySelect));
            self.search(false);
        });

        this.$searchButton.click(function (e) {
            e.preventDefault();
            self.search();
            return false;
        });
        this.$searchTextBox.keypress(event => {
            var keycode = (event.keyCode ? event.keyCode : event.which);
            if (keycode == 13) {
                self.search();
                return false;
            }
        });

        this.$assetsDiv.scroll(() => {
            if (!self.moreSearchPagesExists) return;
            var height = self.$assets.height();
            var outerHeight = self.$assetsDiv.height();
            var scrollTop = self.$assetsDiv.scrollTop();
            const loadMargin = 100;
            if (scrollTop + outerHeight + loadMargin > height) {
                self.search(true);
            }
        });

        //initial search
        await this.search();
    }
    private updateHeight() {
        var categoryDivHeight = this.categoryFilterUI.getCategoryComponentHeight();
        var otherComponentsHeight = 165;
        this.$assetsDiv.css("height", "calc(100vh - " + (categoryDivHeight + otherComponentsHeight) + "px)");
    }
    /**
    * Sets the current sort order. If set to anything else than the default sort order, the order by is locked to the set value
    * @param orderBy the order to sort by
    */
    setSearchOrderBy(orderBy) {
        if (orderBy !== this.getDefaultSearchOrder()) {
            this.overrideDefaultOrderBy = orderBy;
        } else {
            this.overrideDefaultOrderBy = null;
        }
    }
    /**
     * Gets the default search order based on the current search settings.
     * If we search for a expression, matched search order is the default "".
     * If not, "modified desc" is the default
     * 
     * @returns the default sort order
     */
    async getDefaultSearchOrder(): Promise<string> {
        var search = this.getValue(this.$searchTextBox);
        var defaultSearchOrder = "modified desc";
        //if we perform a search and the search engine is enabled, order by match
        if (search && await this.broker.mediaServiceUtil.isSearchEngineEnabled()) {
            defaultSearchOrder = "";
        }
        return defaultSearchOrder;
    }
    ensureMatchSearchOrderExists(ensure: boolean) {
        var firstChild = this.$searchOrderBySelect.children().first()
        var firstChildIsMatchSearchOrder = firstChild.val() === "";
        if (ensure == firstChildIsMatchSearchOrder) return;
        if (ensure) {
            this.$searchOrderBySelect.prepend("<option value=''>" + this.i18n("order-by-match") + "</option>");
        } else {
            this.$searchOrderBySelect.children().first().remove();
        }
    }
    /**
     * Calculates the current active search order.
     * If the currently selected sort order (in the ui) does not match, update the UI accordingly
     * @returns the current sort order
     */
    async getCurrentActiveSearchOrder(): Promise<string> {
        //Calculate sort order
        var order = this.getValue(this.$searchOrderBySelect);
        var newOrder = order;
        var defaultSearchOrder = await this.getDefaultSearchOrder();
        this.ensureMatchSearchOrderExists(defaultSearchOrder === "");
        //if orderBy has explicitly been set, use that
        if (this.overrideDefaultOrderBy) {
            newOrder = this.overrideDefaultOrderBy;
            //else fall back to default search order
        } else if (order != defaultSearchOrder) {
            newOrder = defaultSearchOrder;
        }
        //update ui if order has changed from input.
        if (newOrder != order) {
            order = newOrder;
            this.setValue(this.$searchOrderBySelect, order);
        }
        return order;
    }

    async search(nextPage?: boolean) {
        var self = this;
        //set to false to stop further searches until we know if we have more pages to show
        this.moreSearchPagesExists = false;
        var search = this.getValue(this.$searchTextBox);
        var order = await this.getCurrentActiveSearchOrder();

        var pageSize = 25;
        if (nextPage) {
            this.currentSearchPage++;
        } else {
            this.currentSearchPage = 0;
            this.searchHits = [];
        }
        this.$searchLoader.show();
        //page is 0, reset search result
        if (this.currentSearchPage === 0)
            $("li:not(#search-loader)", self.$assets).remove();
        this.alert(self.i18n("searching"), "info");
        var skip = this.currentSearchPage * pageSize;
        var q: ImageVault.IAssetRequest = {
            $search: search,
            $filter: "isOrganized",
            $top: pageSize,
            $skip: skip,
            $count: true,
            $select: "id,conversions,name,symbols,contentType,metadata,agreementId,vaultId,modified,modifiedBy"
        };
        var catId = this.categoryFilterUI.getSelectedCategories();
        if (catId && catId.length) {
            catId.map((id) => { q.$filter += " and categoryId/any(c: c eq " + id + ")" });
        }
        var version = await this.settings.dataStore.getVersion();
        //support for categories where added in 5.11.147
        if (VersionUtil.verifyMinVersion(version, "5.11.147")) {
            q.$select += ",categories";
        }
        if (order) {
            q.$orderby = order;
        }
        try {
            var r = await this.broker.mediaServiceUtil.search(q, catId);

            var $ul = self.$assets;

            var hits = r["@odata.count"];
            if (hits === 0) {
                self.alert(self.i18n("search-returned-0-results"), "warning");
                self.$searchLoader.hide();
                return;
            }
            if (!r.value || !r.value.length) {
                self.alert(self.i18n("search-yielded-no-additional-results"));
                self.$searchLoader.hide();
                return;
            }
            this.searchHits = this.searchHits.concat(r.value);
            var accessToken = self.settings.client.getCachedAccessToken();
            var imageQs = "?ivauthorization=" + encodeURIComponent(accessToken)
            for (var i = 0; i < r.value.length; i++) {
                var item = r.value[i];
                var name = item.name;
                var conversionInfo = self.findConversionAndResize(item, 200, imageQs);
                //if expired symbol exist, remove soonexpired and valid
                if (item.symbols.filter((s) => s.symbol === "expired").length) {
                    item.symbols = item.symbols.filter((s) => { return s.symbol !== "soonexpired" && s.symbol !== "valid" });
                    //if soonexpired symbol exist, remove valid
                } else if (item.symbols.filter((s) => s.symbol === "soonexpired").length) {
                    item.symbols = item.symbols.filter((s) => { return s.symbol !== "valid" });
                }
                var symbolHtml = item.symbols.map((s) => {
                    var description = "";
                    if (s.info && s.info.length) {
                        description = "<div class='symbol-description'>" + s.info.map(d => { return "<strong>" + ImageVault.Util.htmlEncode(d.name) + "</strong><p>" + ImageVault.Util.htmlEncode(d.value) + "</p>" }).join("<br/>") + "</div>";
                    }
                    return "<div class='symbol symbol-" + s.symbol + "'><i class='icon icon-" + s.symbol + "'></i>" + description + "</div>"
                }).join("");
                $("<li data-id='" + item.id + "'><div class='symbols'>" + symbolHtml + "</div><img src='" + ImageVault.Util.htmlEncode(conversionInfo.url) + "' width='" + conversionInfo.width + "' height='" + conversionInfo.height + "' title='" + ImageVault.Util.htmlEncode(name) + "'/></li>")
                    .insertBefore(self.$searchLoader);
            }

            self.alert(self.i18n("displaying-first-x-assets-of-y-hits", (skip + r.value.length), hits), "success");
            //if more hits is available, make sure that new pages can be loaded
            if (hits > skip + pageSize) {
                self.moreSearchPagesExists = true;
            }
            //unbind event first to prevent it from firing multiple times
            $ul.off("click").on("click", "li", function () {
                var $li = $(this);
                if ($li.is("#search-loader")) return;
                var id = $li.data("id");
                $("li", self.$assets).removeClass("selected");
                $li.addClass("selected");
                self.alert(self.i18n("asset-x-selected", id), "info");
                var index = self.$assets.children().index(this);
                var asset = self.searchHits[index];
                self.broker.selectAsset(asset);
            });
            self.$searchLoader.hide();
        } catch (r) {
            self.$searchLoader.hide();
            self.error(self.i18n("error-performing-search"), r);
        }
    }
    findConversionAndResize(asset, targetSize, imageQs) {
        if (!imageQs) {
            var accessToken = this.settings.client.getCachedAccessToken();
            imageQs = "?ivauthorization=" + encodeURIComponent(accessToken)
        }
        var conversion = this.findClosestConversion(asset.conversions, targetSize);
        if (!conversion) return null;
        //add auth param to allow authorized url:s
        var url = this.settings.config.ivUrl + "apiv2/mediastreamservice" + conversion.url + imageQs;
        var ar = 1.0 * conversion.width / conversion.height;
        var targetSize, height, width;
        if (ar < 1) { //portrait
            height = targetSize;
            width = Math.round(ar * height);
        } else {
            width = targetSize;
            height = Math.round(targetSize / ar);
        }
        return { conversion: conversion, url: url, width: width, height: height };
    }

    findClosestConversion(conversions, width) {
        if (conversions.length === 0) return null;
        var closestConversion = conversions[0];
        var closestDiff = Math.abs(conversions[0].width - width);
        for (var i = 1; i < conversions.length; i++) {
            var newConversion = conversions[i];
            var newDiff = Math.abs(newConversion.width - width);
            if (closestDiff < newDiff) continue;
            closestConversion = newConversion;
            closestDiff = newDiff;
        }
        return closestConversion;
    }

}